@@ -186,27 +186,6 @@ bits_image_fetch_alpha_pixels (bits_image_t *image, uint32_t *buffer, int n_pixe
 static void
 bits_image_fetch_pixels_src_clip (bits_image_t *image, uint32_t *buffer, int n_pixels)
 {
-    if (image->common.src_clip != &(image->common.full_region) &&
-	!pixman_region32_equal (image->common.src_clip, &(image->common.full_region)))
-    {
-	int32_t *coords = (int32_t *)buffer;
-	int i;
-
-	for (i = 0; i < n_pixels; ++i)
-	{
-	    int32_t x = coords[0];
-	    int32_t y = coords[1];
-
-	    if (!pixman_region32_contains_point (image->common.src_clip, x, y, NULL))
-	    {
-		coords[0] = 0xffffffff;
-		coords[1] = 0xffffffff;
-	    }
-
-	    coords += 2;
-	}
-    }
-
     bits_image_fetch_alpha_pixels (image, buffer, n_pixels);
 }
 
@@ -657,12 +636,88 @@ bits_image_fetch_solid_64 (bits_image_t * image,
 	*(buffer++) = color[0];
 }
 
+static void
+bits_image_fetch_untransformed_repeat_none (bits_image_t *image, pixman_bool_t wide,
+					    int x, int y, int width,
+					    uint32_t *buffer)
+{
+    uint32_t w;
+
+    if (y < 0 || y >= image->height)
+    {
+	memset (buffer, 0, width * sizeof (uint32_t));
+	return;
+    }
+
+    if (x < 0)
+    {
+	w = MIN (width, -x);
+	
+	memset (buffer, 0, w * (wide? 8 : 4));
+	
+	width -= w;
+	buffer += w;
+	x += w;
+    }
+    
+    if (x < image->width)
+    {
+	w = MIN (width, image->width - x);
+	
+	if (wide)
+	    image->fetch_scanline_raw_64 (image, x, y, w, (uint64_t *)buffer);
+	else
+	    image->fetch_scanline_raw_32 (image, x, y, w, buffer);
+	
+	width -= w;
+	buffer += w;
+	x += w;
+    }
+
+    memset (buffer, 0, width * (wide? 8 : 4));
+}
+
+static void
+bits_image_fetch_untransformed_repeat_normal (bits_image_t *image, pixman_bool_t wide,
+					      int x, int y, int width,
+					      uint32_t *buffer)
+{
+    uint32_t w;
+    
+    while (y < 0)
+	y += image->height;
+    while (y >= image->height)
+	y -= image->height;
+    
+    while (width)
+    {
+	while (x < 0)
+	    x += image->width;
+	while (x >= image->width)
+	    x -= image->width;
+	
+	w = MIN (width, image->width - x);
+	
+	if (wide)
+	    image->fetch_scanline_raw_64 (image, x, y, w, (uint64_t *)buffer);
+	else
+	    image->fetch_scanline_raw_32 (image, x, y, w, buffer);
+	
+	buffer += w;
+	x += w;
+	width -= w;
+    }
+}
+
 static void
 bits_image_fetch_untransformed_32 (bits_image_t * image,
 				   int x, int y, int width,
 				   uint32_t *buffer, uint32_t *mask, uint32_t maskBits)
 {
-    image->fetch_scanline_raw_32 (image, x, y, width, buffer);
+    if (image->common.repeat == PIXMAN_REPEAT_NONE)
+	bits_image_fetch_untransformed_repeat_none (image, FALSE, x, y, width, buffer);
+    else
+	bits_image_fetch_untransformed_repeat_normal (image, FALSE, x, y, width, buffer);
 }
 
 static void
@@ -670,7 +725,10 @@ bits_image_fetch_untransformed_64 (bits_image_t * image,
 				   int x, int y, int width,
 				   uint64_t *buffer, void *unused, uint32_t unused2)
 {
-    image->fetch_scanline_raw_64 (image, x, y, width, buffer);
+    if (image->common.repeat == PIXMAN_REPEAT_NONE)
+	bits_image_fetch_untransformed_repeat_none (image, FALSE, x, y, width, (uint32_t *)buffer);
+    else
+	bits_image_fetch_untransformed_repeat_normal (image, FALSE, x, y, width, (uint32_t *)buffer);
 }
 
 static void
@@ -694,8 +752,8 @@ bits_image_property_changed (pixman_image_t *image)
     }
     else if (!bits->common.transform &&
 	     bits->common.filter != PIXMAN_FILTER_CONVOLUTION &&
-	     bits->common.repeat != PIXMAN_REPEAT_PAD &&
-	     bits->common.repeat != PIXMAN_REPEAT_REFLECT)
+	     (bits->common.repeat == PIXMAN_REPEAT_NONE ||
+	      bits->common.repeat == PIXMAN_REPEAT_NORMAL))
     {
 	image->common.get_scanline_64 = (scanFetchProc)bits_image_fetch_untransformed_64;
 	image->common.get_scanline_32 = (scanFetchProc)bits_image_fetch_untransformed_32;
@@ -810,10 +868,6 @@ pixman_image_create_bits (pixman_format_code_t  format,
 									*/
     image->bits.indexed = NULL;
     
-    pixman_region32_fini (&image->common.full_region);
-    pixman_region32_init_rect (&image->common.full_region, 0, 0,
-			       image->bits.width, image->bits.height);
-    
     image->common.property_changed = bits_image_property_changed;
     
     bits_image_property_changed (image);
