@@ -7,6 +7,7 @@
 
 #include "pixman.h"
 #include <time.h>
+#include <assert.h>
 
 #ifndef FALSE
 #define FALSE 0
@@ -152,9 +153,7 @@ typedef struct point point_t;
  */
 
 #define FASTCALL
-typedef FASTCALL void (*CombineMaskU32) (uint32_t *src, const uint32_t *mask, int width);
-typedef FASTCALL void (*CombineFuncU32) (uint32_t *dest, const uint32_t *src, int width);
-typedef FASTCALL void (*CombineFuncC32) (uint32_t *dest, uint32_t *src, uint32_t *mask, int width);
+typedef FASTCALL void (*CombineFunc32) (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width);
 typedef FASTCALL void (*fetchProc32)(bits_image_t *pict, int x, int y, int width,
                                      uint32_t *buffer);
 typedef FASTCALL uint32_t (*fetchPixelProc32)(bits_image_t *pict, int offset, int line);
@@ -162,9 +161,7 @@ typedef FASTCALL void (*storeProc32)(pixman_image_t *, uint32_t *bits,
                                      const uint32_t *values, int x, int width,
                                      const pixman_indexed_t *);
 
-typedef FASTCALL void (*CombineMaskU64) (uint64_t *src, const uint64_t *mask, int width);
-typedef FASTCALL void (*CombineFuncU64) (uint64_t *dest, const uint64_t *src, int width);
-typedef FASTCALL void (*CombineFuncC64) (uint64_t *dest, uint64_t *src, uint64_t *mask, int width);
+typedef FASTCALL void (*CombineFunc64) (uint64_t *dest, const uint64_t *src, const uint64_t *mask, int width);
 typedef FASTCALL void (*fetchProc64)(bits_image_t *pict, int x, int y, int width,
                                      uint64_t *buffer);
 typedef FASTCALL uint64_t (*fetchPixelProc64)(bits_image_t *pict, int offset, int line);
@@ -187,20 +184,8 @@ typedef struct _FbComposeData {
     uint16_t	 height;
 } FbComposeData;
 
-typedef struct _FbComposeFunctions32 {
-    CombineFuncU32 *combineU;
-    CombineFuncC32 *combineC;
-    CombineMaskU32 combineMaskU;
-} FbComposeFunctions32;
-
-typedef struct _FbComposeFunctions64 {
-    CombineFuncU64 *combineU;
-    CombineFuncC64 *combineC;
-    CombineMaskU64 combineMaskU;
-} FbComposeFunctions64;
-
-extern FbComposeFunctions32 pixman_composeFunctions;
-extern FbComposeFunctions64 pixman_composeFunctions64;
+typedef void (* fetch_pixels_32_t) (bits_image_t *image, uint32_t *buffer, int n_pixels);
+typedef void (* fetch_pixels_64_t) (bits_image_t *image, uint64_t *buffer, int n_pixels);
 
 void pixman_composite_rect_general_accessors (const FbComposeData *data,
                                               void *src_buffer,
@@ -210,58 +195,26 @@ void pixman_composite_rect_general_accessors (const FbComposeData *data,
 void pixman_composite_rect_general (const FbComposeData *data);
 
 fetchProc32 pixman_fetchProcForPicture32 (bits_image_t *);
-fetchPixelProc32 pixman_fetchPixelProcForPicture32 (bits_image_t *);
+fetch_pixels_32_t pixman_fetchPixelProcForPicture32 (bits_image_t *);
 storeProc32 pixman_storeProcForPicture32 (bits_image_t *);
 fetchProc32 pixman_fetchProcForPicture32_accessors (bits_image_t *);
-fetchPixelProc32 pixman_fetchPixelProcForPicture32_accessors (bits_image_t *);
+fetch_pixels_32_t pixman_fetchPixelProcForPicture32_accessors (bits_image_t *);
 storeProc32 pixman_storeProcForPicture32_accessors (bits_image_t *);
 
 fetchProc64 pixman_fetchProcForPicture64 (bits_image_t *);
-fetchPixelProc64 pixman_fetchPixelProcForPicture64 (bits_image_t *);
+fetch_pixels_64_t pixman_fetchPixelProcForPicture64 (bits_image_t *);
 storeProc64 pixman_storeProcForPicture64 (bits_image_t *);
 fetchProc64 pixman_fetchProcForPicture64_accessors (bits_image_t *);
-fetchPixelProc64 pixman_fetchPixelProcForPicture64_accessors (bits_image_t *);
+fetch_pixels_64_t pixman_fetchPixelProcForPicture64_accessors (bits_image_t *);
 storeProc64 pixman_storeProcForPicture64_accessors (bits_image_t *);
 
 void pixman_expand(uint64_t *dst, const uint32_t *src, pixman_format_code_t, int width);
 void pixman_contract(uint32_t *dst, const uint64_t *src, int width);
 
-void pixmanFetchSourcePict(source_image_t *, int x, int y, int width,
+void pixmanFetchGradient (gradient_t *, int x, int y, int width,
                            uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
-void pixmanFetchSourcePict64(source_image_t *, int x, int y, int width,
-                             uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
-
-void fbFetchTransformed(bits_image_t *, int x, int y, int width,
-                        uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
-void fbStoreExternalAlpha(bits_image_t *, int x, int y, int width,
-                          uint32_t *buffer);
-void fbFetchExternalAlpha(bits_image_t *, int x, int y, int width,
-                          uint32_t *buffer, uint32_t *mask, uint32_t maskBits);
-
-void fbFetchTransformed_accessors(bits_image_t *, int x, int y, int width,
-                                  uint32_t *buffer, uint32_t *mask,
-                                  uint32_t maskBits);
-void fbStoreExternalAlpha_accessors(bits_image_t *, int x, int y, int width,
-                                    uint32_t *buffer);
-void fbFetchExternalAlpha_accessors(bits_image_t *, int x, int y, int width,
-                                    uint32_t *buffer, uint32_t *mask,
-                                    uint32_t maskBits);
-
-void fbFetchTransformed64(bits_image_t *, int x, int y, int width,
-                          uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
-void fbStoreExternalAlpha64(bits_image_t *, int x, int y, int width,
-                            uint64_t *buffer);
-void fbFetchExternalAlpha64(bits_image_t *, int x, int y, int width,
-                            uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
-
-void fbFetchTransformed64_accessors(bits_image_t *, int x, int y, int width,
-                                    uint64_t *buffer, uint64_t *mask,
-                                    uint32_t maskBits);
-void fbStoreExternalAlpha64_accessors(bits_image_t *, int x, int y, int width,
-                                      uint64_t *buffer);
-void fbFetchExternalAlpha64_accessors(bits_image_t *, int x, int y, int width,
-                                      uint64_t *buffer, uint64_t *mask,
-                                      uint32_t maskBits);
+void _pixman_image_get_scanline_64_generic (pixman_image_t * pict, int x, int y, int width,
+					    uint64_t *buffer, uint64_t *mask, uint32_t maskBits);
 
 /* end */
 
@@ -280,22 +233,75 @@ typedef enum
 {
     SOURCE_IMAGE_CLASS_UNKNOWN,
     SOURCE_IMAGE_CLASS_HORIZONTAL,
-    SOURCE_IMAGE_CLASS_VERTICAL
+    SOURCE_IMAGE_CLASS_VERTICAL,
 } source_pict_class_t;
 
+typedef void (*scanStoreProc)(bits_image_t *img, int x, int y, int width, uint32_t *buffer);
+typedef void (*scanFetchProc)(pixman_image_t *, int, int, int, uint32_t *,
+			      uint32_t *, uint32_t);
+
+source_pict_class_t _pixman_image_classify (pixman_image_t *image,
+					    int             x,
+					    int             y,
+					    int             width,
+					    int             height);
+
+void
+_pixman_image_get_scanline_32 (pixman_image_t *image, int x, int y, int width,
+			       uint32_t *buffer, uint32_t *mask, uint32_t mask_bits);
+
+/* Even thought the type of buffer is uint32_t *, the function actually expects
+ * a uint64_t *buffer.
+ */
+void
+_pixman_image_get_scanline_64 (pixman_image_t *image, int x, int y, int width,
+			       uint32_t *buffer, uint32_t *unused, uint32_t unused2);
+
+void
+_pixman_image_store_scanline_32 (bits_image_t *image, int x, int y, int width,
+				 uint32_t *buffer);
+void
+_pixman_image_fetch_pixels (bits_image_t *image, uint32_t *buffer, int n_pixels);
+
+/* Even thought the type of buffer is uint32_t *, the function actually expects
+ * a uint64_t *buffer.
+ */
+void
+_pixman_image_store_scanline_64 (bits_image_t *image, int x, int y, int width,
+				 uint32_t *buffer);
+
+pixman_image_t *
+_pixman_image_allocate (void);
+
+pixman_bool_t
+_pixman_init_gradient (gradient_t     *gradient,
+		       const pixman_gradient_stop_t *stops,
+		       int	       n_stops);
+void
+_pixman_image_reset_clip_region (pixman_image_t *image);
+
 struct point
 {
     int16_t x, y;
 };
 
+typedef source_pict_class_t (* classify_func_t) (pixman_image_t *image,
+						 int             x,
+						 int             y,
+						 int             width,
+						 int             height);
+typedef void (* property_changed_func_t)        (pixman_image_t *image);
+
 struct image_common
 {
     image_type_t		type;
     int32_t			ref_count;
-    pixman_region32_t		full_region;
     pixman_region32_t		clip_region;
-    pixman_region32_t	       *src_clip;
-    pixman_bool_t               has_client_clip;
+    pixman_bool_t		have_clip_region;	/* FALSE if there is no clip */
+    pixman_bool_t		client_clip;	/* Whether the source clip was set by a client */
+    pixman_bool_t		clip_sources;		/* Whether the clip applies when
+							 * the image is used as a source
+							 */
     pixman_transform_t	       *transform;
     pixman_repeat_t		repeat;
     pixman_filter_t		filter;
@@ -306,6 +312,13 @@ struct image_common
     pixman_bool_t		component_alpha;
     pixman_read_memory_func_t	read_func;
     pixman_write_memory_func_t	write_func;
+    classify_func_t		classify;
+    property_changed_func_t	property_changed;
+    scanFetchProc		get_scanline_32;
+    scanFetchProc		get_scanline_64;
+
+    pixman_image_destroy_func_t destroy_func;
+    void *			destroy_data;
 };
 
 struct source_image
@@ -373,6 +386,18 @@ struct bits_image
     uint32_t *			bits;
     uint32_t *			free_me;
     int				rowstride; /* in number of uint32_t's */
+
+    fetch_pixels_32_t		fetch_pixels_32;
+    fetch_pixels_64_t		fetch_pixels_64;
+
+    scanStoreProc		store_scanline_32;
+    scanStoreProc		store_scanline_64;
+
+    storeProc32			store_scanline_raw_32;
+    storeProc64			store_scanline_raw_64;
+
+    fetchProc32			fetch_scanline_raw_32;
+    fetchProc64			fetch_scanline_raw_64;
 };
 
 union pixman_image
@@ -380,6 +405,7 @@ union pixman_image
     image_type_t		type;
     image_common_t		common;
     bits_image_t		bits;
+    source_image_t		source;
     gradient_t			gradient;
     linear_gradient_t		linear;
     conical_gradient_t		conical;
@@ -387,6 +413,39 @@ union pixman_image
     solid_fill_t		solid;
 };
 
+/* Gradient walker
+ */
+typedef struct
+{
+    uint32_t        left_ag;
+    uint32_t        left_rb;
+    uint32_t        right_ag;
+    uint32_t        right_rb;
+    int32_t       left_x;
+    int32_t       right_x;
+    int32_t       stepper;
+
+    pixman_gradient_stop_t	*stops;
+    int                      num_stops;
+    unsigned int             spread;
+
+    int		  need_reset;
+} GradientWalker;
+
+void
+_pixman_gradient_walker_init (GradientWalker  *walker,
+			      gradient_t      *gradient,
+			      unsigned int     spread);
+
+void
+_pixman_gradient_walker_reset (GradientWalker       *walker,
+			       pixman_fixed_32_32_t  pos);
+
+uint32_t
+_pixman_gradient_walker_pixel (GradientWalker       *walker,
+			       pixman_fixed_32_32_t  x);
+
+
 
 #define LOG2_BITMAP_PAD 5
 #define FB_STIP_SHIFT	LOG2_BITMAP_PAD
@@ -500,11 +559,20 @@ union pixman_image
 
 #define MOD(a,b) ((a) < 0 ? ((b) - ((-(a) - 1) % (b))) - 1 : (a) % (b))
 
+/* Divides two fixed-point numbers and returns an integer */
 #define DIV(a,b) ((((a) < 0) == ((b) < 0)) ? (a) / (b) :		\
 		  ((a) - (b) + 1 - (((b) < 0) << 1)) / (b))
 
 #define CLIP(a,b,c) ((a) < (b) ? (b) : ((a) > (c) ? (c) : (a)))
 
+#ifndef MIN
+#  define MIN(a,b) ((a < b)? a : b)
+#endif
+
+#ifndef MAX
+#  define MAX(a,b) ((a > b)? a : b)
+#endif
+
 #if 0
 /* FIXME: the MOD macro above is equivalent, but faster I think */
 #define mod(a,b) ((b) == 1 ? 0 : (a) >= 0 ? (a) % (b) : (b) - (-a) % (b))
@@ -515,40 +583,6 @@ union pixman_image
  * where Fetch4 doesn't have a READ
  */
 
-#if 0
-/* Framebuffer access support macros */
-#define ACCESS_MEM(code)						\
-    do {								\
-	const image_common_t *const com__ =				\
-	    (image_common_t *)image;					\
-									\
-	if (!com__->read_func && !com__->write_func)			\
-	{								\
-	    const int do_access__ = 0;					\
-	    const pixman_read_memory_func_t read_func__ = NULL;		\
-	    const pixman_write_memory_func_t write_func__ = NULL;	\
-	    (void)read_func__;						\
-	    (void)write_func__;						\
-	    (void)do_access__;						\
-									\
-	    {code}							\
-	}								\
-	else								\
-	{								\
-	    const int do_access__ = 1;					\
-	    const pixman_read_memory_func_t read_func__ =		\
-		com__->read_func;					\
-	    const pixman_write_memory_func_t write_func__ =		\
-		com__->write_func;					\
-	    (void)read_func__;						\
-	    (void)write_func__;						\
-	    (void)do_access__;						\
-	    								\
-	    {code}							\
-	}								\
-    } while (0)
-#endif
-
 #ifdef PIXMAN_FB_ACCESSORS
 
 #define ACCESS(sym) sym##_accessors
@@ -582,9 +616,9 @@ union pixman_image
 
 #define READ(img, ptr)		(*(ptr))
 #define WRITE(img, ptr, val)	(*(ptr) = (val))
-#define MEMCPY_WRAPPED(img, dst, src, size)					\
+#define MEMCPY_WRAPPED(img, dst, src, size)				\
     memcpy(dst, src, size)
-#define MEMSET_WRAPPED(img, dst, val, size)					\
+#define MEMSET_WRAPPED(img, dst, val, size)				\
     memset(dst, val, size)
 
 #endif
@@ -609,7 +643,7 @@ union pixman_image
 		(res) = READ(img, (uint32_t *)bits__);			\
 		break;							\
 	    case 24:							\
-		(res) = Fetch24(img, (uint8_t *) bits__);			\
+		(res) = Fetch24(img, (uint8_t *) bits__);		\
 		break;							\
 	    case 16:							\
 		(res) = READ(img, (uint16_t *) bits__);			\
@@ -754,7 +788,7 @@ void pixman_region_internal_set_static_pointers (pixman_box16_t *empty_box,
 						 pixman_region16_data_t *empty_data,
 						 pixman_region16_data_t *broken_data);
 
-#ifdef PIXMAN_TIMING
+#ifdef PIXMAN_TIMERS
 
 /* Timing */
 static inline uint64_t
@@ -799,6 +833,228 @@ void pixman_timer_register (PixmanTimer *timer);
         timer##tname.total += OIL_STAMP() - begin##tname;		\
     }
 
-#endif /* PIXMAN_TIMING */
+#endif /* PIXMAN_TIMERS */
+
+typedef struct pixman_implementation_t pixman_implementation_t;
+
+typedef void (* pixman_combine_32_func_t) (pixman_implementation_t *	imp,
+					   pixman_op_t			op,
+					   uint32_t *			dest,
+					   const uint32_t *		src,
+					   const uint32_t *		mask,
+					   int				width);
+
+typedef void (* pixman_combine_64_func_t) (pixman_implementation_t *	imp,
+					   pixman_op_t			op,
+					   uint64_t *			dest,
+					   const uint64_t *		src,
+					   const uint64_t *		mask,
+					   int				width);
+
+typedef void (* pixman_composite_func_t)  (pixman_implementation_t *	imp,
+					   pixman_op_t			op,
+					   pixman_image_t *		src,
+					   pixman_image_t *		mask,
+					   pixman_image_t *		dest,
+					   int32_t			src_x,
+					   int32_t			src_y,
+					   int32_t			mask_x,
+					   int32_t			mask_y,
+					   int32_t			dest_x,
+					   int32_t			dest_y,
+					   int32_t			width,
+					   int32_t			height);
+typedef pixman_bool_t (* pixman_blt_func_t) (pixman_implementation_t *	imp,
+					     uint32_t *			src_bits,
+					     uint32_t *			dst_bits,
+					     int			src_stride,
+					     int			dst_stride,
+					     int			src_bpp,
+					     int			dst_bpp,
+					     int			src_x,
+					     int			src_y,
+					     int			dst_x,
+					     int			dst_y,
+					     int			width,
+					     int			height);
+typedef pixman_bool_t (* pixman_fill_func_t) (pixman_implementation_t *imp,
+					      uint32_t *bits,
+					      int stride,
+					      int bpp,
+					      int x,
+					      int y,
+					      int width,
+					      int height,
+					      uint32_t xor);
+
+void
+_pixman_walk_composite_region (pixman_implementation_t *imp,
+			       pixman_op_t op,
+			       pixman_image_t * pSrc,
+			       pixman_image_t * pMask,
+			       pixman_image_t * pDst,
+			       int16_t xSrc,
+			       int16_t ySrc,
+			       int16_t xMask,
+			       int16_t yMask,
+			       int16_t xDst,
+			       int16_t yDst,
+			       uint16_t width,
+			       uint16_t height,
+			       pixman_composite_func_t compositeRect);
+
+void _pixman_setup_combiner_functions_32 (pixman_implementation_t *imp);
+void _pixman_setup_combiner_functions_64 (pixman_implementation_t *imp);
+
+/* These "formats" both have depth 0, so they
+ * will never clash with any real ones
+ */
+#define PIXMAN_null		PIXMAN_FORMAT(0,0,0,0,0,0)
+#define PIXMAN_solid		PIXMAN_FORMAT(0,1,0,0,0,0)
+
+#define NEED_COMPONENT_ALPHA		(1 << 0)
+#define NEED_PIXBUF			(1 << 1)
+#define NEED_SOLID_MASK		        (1 << 2)
+
+typedef struct
+{
+    pixman_op_t			op;
+    pixman_format_code_t	src_format;
+    pixman_format_code_t	mask_format;
+    pixman_format_code_t	dest_format;
+    pixman_composite_func_t	func;
+    uint32_t			flags;
+} FastPathInfo;
+
+struct pixman_implementation_t
+{
+    pixman_implementation_t *	toplevel;
+    pixman_implementation_t *	delegate;
+
+    pixman_composite_func_t	composite;
+    pixman_blt_func_t		blt;
+    pixman_fill_func_t		fill;
+    
+    pixman_combine_32_func_t	combine_32[PIXMAN_OP_LAST];
+    pixman_combine_32_func_t	combine_32_ca[PIXMAN_OP_LAST];
+    pixman_combine_64_func_t	combine_64[PIXMAN_OP_LAST];
+    pixman_combine_64_func_t	combine_64_ca[PIXMAN_OP_LAST];
+};
+
+pixman_implementation_t *
+_pixman_implementation_create (pixman_implementation_t *delegate);
+
+void
+_pixman_implementation_combine_32 (pixman_implementation_t *	imp,
+				   pixman_op_t			op,
+				   uint32_t *			dest,
+				   const uint32_t *		src,
+				   const uint32_t *		mask,
+				   int				width);
+void
+_pixman_implementation_combine_64 (pixman_implementation_t *	imp,
+				   pixman_op_t			op,
+				   uint64_t *			dest,
+				   const uint64_t *		src,
+				   const uint64_t *		mask,
+				   int				width);
+void
+_pixman_implementation_combine_32_ca (pixman_implementation_t *	imp,
+				      pixman_op_t		op,
+				      uint32_t *		dest,
+				      const uint32_t *		src,
+				      const uint32_t *		mask,
+				      int			width);
+void
+_pixman_implementation_combine_64_ca (pixman_implementation_t *	imp,
+				      pixman_op_t		op,
+				      uint64_t *		dest,
+				      const uint64_t *		src,
+				      const uint64_t *		mask,
+				      int			width);
+void
+_pixman_implementation_composite (pixman_implementation_t *	imp,
+				  pixman_op_t			op,
+				  pixman_image_t *		src,
+				  pixman_image_t *		mask,
+				  pixman_image_t *		dest,
+				  int32_t			src_x,
+				  int32_t			src_y,
+				  int32_t			mask_x,
+				  int32_t			mask_y,
+				  int32_t			dest_x,
+				  int32_t			dest_y,
+				  int32_t			width,
+				  int32_t			height);
+
+pixman_bool_t
+_pixman_implementation_blt (pixman_implementation_t *	imp,
+			    uint32_t *			src_bits,
+			    uint32_t *			dst_bits,
+			    int				src_stride,
+			    int				dst_stride,
+			    int				src_bpp,
+			    int				dst_bpp,
+			    int				src_x,
+			    int				src_y,
+			    int				dst_x,
+			    int				dst_y,
+			    int				width,
+			    int				height);
+pixman_bool_t
+_pixman_implementation_fill (pixman_implementation_t *   imp,
+			     uint32_t *bits,
+			     int stride,
+			     int bpp,
+			     int x,
+			     int y,
+			     int width,
+			     int height,
+			     uint32_t xor);
+    
+/* Specific implementations */
+pixman_implementation_t *
+_pixman_implementation_create_general (void);
+pixman_implementation_t *
+_pixman_implementation_create_fast_path (void);
+#ifdef USE_MMX
+pixman_implementation_t *
+_pixman_implementation_create_mmx (void);
+#endif
+#ifdef USE_SSE2
+pixman_implementation_t *
+_pixman_implementation_create_sse2 (void);
+#endif
+#ifdef USE_ARM_SIMD
+pixman_implementation_t *
+_pixman_implementation_create_arm_simd (void);
+#endif
+#ifdef USE_ARM_NEON
+pixman_implementation_t *
+_pixman_implementation_create_arm_neon (void);
+#endif
+#ifdef USE_VMX
+pixman_implementation_t *
+_pixman_implementation_create_vmx (void);
+#endif
+
+pixman_bool_t
+_pixman_run_fast_path (const FastPathInfo *paths,
+		       pixman_implementation_t *imp,
+		       pixman_op_t op,
+		       pixman_image_t *src,
+		       pixman_image_t *mask,
+		       pixman_image_t *dest,
+		       int32_t src_x,
+		       int32_t src_y,
+		       int32_t mask_x,
+		       int32_t mask_y,
+		       int32_t dest_x,
+		       int32_t dest_y,
+		       int32_t width,
+		       int32_t height);
+    
+pixman_implementation_t *
+_pixman_choose_implementation (void);
 
 #endif /* PIXMAN_PRIVATE_H */
