@@ -527,39 +527,29 @@ pixman_format_supported_source (pixman_format_code_t format)
     }
 }
 
-void
-_pixman_walk_composite_region (pixman_implementation_t *imp,
-			      pixman_op_t op,
-			      pixman_image_t * pSrc,
-			      pixman_image_t * pMask,
-			      pixman_image_t * pDst,
-			      int16_t xSrc,
-			      int16_t ySrc,
-			      int16_t xMask,
-			      int16_t yMask,
-			      int16_t xDst,
-			      int16_t yDst,
-			      uint16_t width,
-			      uint16_t height,
-			      pixman_bool_t srcRepeat,
-			      pixman_bool_t maskRepeat,
-			      pixman_composite_func_t compositeRect)
+static void
+walk_region_internal (pixman_implementation_t *imp,
+		      pixman_op_t op,
+		      pixman_image_t * pSrc,
+		      pixman_image_t * pMask,
+		      pixman_image_t * pDst,
+		      int16_t xSrc,
+		      int16_t ySrc,
+		      int16_t xMask,
+		      int16_t yMask,
+		      int16_t xDst,
+		      int16_t yDst,
+		      uint16_t width,
+		      uint16_t height,
+		      pixman_bool_t srcRepeat,
+		      pixman_bool_t maskRepeat,
+		      pixman_region32_t *region,
+		      pixman_composite_func_t compositeRect)
 {
-    int		    n;
+    int n;
     const pixman_box32_t *pbox;
-    int		    w, h, w_this, h_this;
-    int		    x_msk, y_msk, x_src, y_src, x_dst, y_dst;
-    pixman_region32_t reg;
-    pixman_region32_t *region;
-
-    pixman_region32_init (&reg);
-    if (!pixman_compute_composite_region32 (&reg, pSrc, pMask, pDst,
-					    xSrc, ySrc, xMask, yMask, xDst, yDst, width, height))
-    {
-	return;
-    }
-
-    region = &reg;
+    int w, h, w_this, h_this;
+    int x_msk, y_msk, x_src, y_src, x_dst, y_dst;
 
     pbox = pixman_region32_rectangles (region, &n);
     while (n--)
@@ -575,6 +565,7 @@ _pixman_walk_composite_region (pixman_implementation_t *imp,
 	    x_src = pbox->x1 - xDst + xSrc;
 	    x_msk = pbox->x1 - xDst + xMask;
 	    x_dst = pbox->x1;
+	    
 	    if (maskRepeat)
 	    {
 		y_msk = MOD (y_msk, pMask->bits.height);
@@ -618,9 +609,43 @@ _pixman_walk_composite_region (pixman_implementation_t *imp,
 	}
 	pbox++;
     }
-    pixman_region32_fini (&reg);
 }
 
+void
+_pixman_walk_composite_region (pixman_implementation_t *imp,
+			       pixman_op_t op,
+			       pixman_image_t * pSrc,
+			       pixman_image_t * pMask,
+			       pixman_image_t * pDst,
+			       int16_t xSrc,
+			       int16_t ySrc,
+			       int16_t xMask,
+			       int16_t yMask,
+			       int16_t xDst,
+			       int16_t yDst,
+			       uint16_t width,
+			       uint16_t height,
+			       pixman_composite_func_t compositeRect)
+{
+    pixman_region32_t region;
+    
+    pixman_region32_init (&region);
+
+    if (pixman_compute_composite_region32 (
+	    &region, pSrc, pMask, pDst, xSrc, ySrc, xMask, yMask, xDst, yDst, width, height))
+    {
+	walk_region_internal (imp, op,
+			      pSrc, pMask, pDst,
+			      xSrc, ySrc, xMask, yMask, xDst, yDst,
+			      width, height, FALSE, FALSE,
+			      &region,
+			      compositeRect);
+    }
+
+    pixman_region32_fini (&region);
+}
+
+    
 static pixman_bool_t
 mask_is_solid (pixman_image_t *mask)
 {
@@ -650,13 +675,13 @@ get_fast_path (const FastPathInfo *fast_paths,
 
     for (info = fast_paths; info->op != PIXMAN_OP_NONE; info++)
     {
-	pixman_bool_t valid_src		= FALSE;
-	pixman_bool_t valid_mask	= FALSE;
+	pixman_bool_t valid_src = FALSE;
+	pixman_bool_t valid_mask = FALSE;
 
 	if (info->op != op)
 	    continue;
 
-	if ((info->src_format == PIXMAN_solid && pixman_image_can_get_solid (pSrc))		||
+	if ((info->src_format == PIXMAN_solid && pixman_image_can_get_solid (pSrc)) ||
 	    (pSrc->type == BITS && info->src_format == pSrc->bits.format))
 	{
 	    valid_src = TRUE;
@@ -665,7 +690,7 @@ get_fast_path (const FastPathInfo *fast_paths,
 	if (!valid_src)
 	    continue;
 
-	if ((info->mask_format == PIXMAN_null && !pMask)			||
+	if ((info->mask_format == PIXMAN_null && !pMask) ||
 	    (pMask && pMask->type == BITS && info->mask_format == pMask->bits.format))
 	{
 	    valid_mask = TRUE;
@@ -698,6 +723,22 @@ get_fast_path (const FastPathInfo *fast_paths,
     return NULL;
 }
 
+static inline pixman_bool_t
+image_covers (pixman_image_t *image, pixman_box32_t *extents)
+{
+    if (image->common.type == BITS && image->common.repeat == PIXMAN_REPEAT_NONE)
+    {
+	if (extents->x1 < 0 || extents->y1 < 0 ||
+	    extents->x2 >= image->bits.width ||
+	    extents->y2 >= image->bits.height)
+	{
+	    return FALSE;
+	}
+    }
+
+    return TRUE;
+}
+
 pixman_bool_t
 _pixman_run_fast_path (const FastPathInfo *paths,
 		       pixman_implementation_t *imp,
@@ -717,15 +758,16 @@ _pixman_run_fast_path (const FastPathInfo *paths,
     pixman_composite_func_t func = NULL;
     pixman_bool_t src_repeat = src->common.repeat == PIXMAN_REPEAT_NORMAL;
     pixman_bool_t mask_repeat = mask && mask->common.repeat == PIXMAN_REPEAT_NORMAL;
-    
+    pixman_bool_t result;
+
     if ((src->type == BITS || pixman_image_can_get_solid (src)) &&
 	(!mask || mask->type == BITS)
-        && !src->common.transform && !(mask && mask->common.transform)
-        && !(mask && mask->common.alpha_map) && !src->common.alpha_map && !dest->common.alpha_map
-        && (src->common.filter != PIXMAN_FILTER_CONVOLUTION)
-        && (src->common.repeat != PIXMAN_REPEAT_PAD)
-        && (src->common.repeat != PIXMAN_REPEAT_REFLECT)
-        && (!mask || (mask->common.filter != PIXMAN_FILTER_CONVOLUTION &&
+	&& !src->common.transform && !(mask && mask->common.transform)
+	&& !(mask && mask->common.alpha_map) && !src->common.alpha_map && !dest->common.alpha_map
+	&& (src->common.filter != PIXMAN_FILTER_CONVOLUTION)
+	&& (src->common.repeat != PIXMAN_REPEAT_PAD)
+	&& (src->common.repeat != PIXMAN_REPEAT_REFLECT)
+	&& (!mask || (mask->common.filter != PIXMAN_FILTER_CONVOLUTION &&
 		      mask->common.repeat != PIXMAN_REPEAT_PAD &&
 		      mask->common.repeat != PIXMAN_REPEAT_REFLECT))
 	&& !src->common.read_func && !src->common.write_func
@@ -747,17 +789,17 @@ _pixman_run_fast_path (const FastPathInfo *paths,
 	    !mask_repeat;
 	
 	info = get_fast_path (paths, op, src, mask, dest, pixbuf);
-
+	
 	if (info)
 	{
 	    func = info->func;
-		
+	    
 	    if (info->src_format == PIXMAN_solid)
 		src_repeat = FALSE;
-
+	    
 	    if (info->mask_format == PIXMAN_solid || info->flags & NEED_SOLID_MASK)
 		mask_repeat = FALSE;
-
+	    
 	    if ((src_repeat			&&
 		 src->bits.width == 1		&&
 		 src->bits.height == 1)	||
@@ -777,18 +819,37 @@ _pixman_run_fast_path (const FastPathInfo *paths,
 	    }
 	}
     }
-
+    
+    result = FALSE;
+    
     if (func)
     {
-	_pixman_walk_composite_region (imp, op,
-				       src, mask, dest,
-				       src_x, src_y, mask_x, mask_y,
-				       dest_x, dest_y,
-				       width, height,
-				       src_repeat, mask_repeat,
-				       func);
-	return TRUE;
+	pixman_region32_t region;
+	pixman_region32_init (&region);
+
+	if (pixman_compute_composite_region32 (
+		&region, src, mask, dest, src_x, src_y, mask_x, mask_y, dest_x, dest_y, width, height))
+	{
+	    pixman_box32_t *extents = pixman_region32_extents (&region);
+	
+	    if (image_covers (src, extents)		   &&
+		(!mask || image_covers (mask, extents)))
+	    {
+		walk_region_internal (imp, op,
+				      src, mask, dest,
+				      src_x, src_y, mask_x, mask_y,
+				      dest_x, dest_y,
+				      width, height,
+				      src_repeat, mask_repeat,
+				      &region,
+				      func);
+	    
+		result = TRUE;
+	    }
+	}
+	    
+	pixman_region32_fini (&region);
     }
     
-    return FALSE;
+    return result;
 }
